<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2005 Larry Adams                                          |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
*/

/*	valid_snmp_device - This function validates that the device is reachable via snmp.
  It first attempts	to utilize the default snmp readstring.  If it's not valid, it
  attempts to find the correct read string and then updates several system
  information variable. it returns the status	of the host (up=true, down=false)
*/
function valid_snmp_device(&$device) {
	/* initialize variable */
	$host_up = FALSE;
	$device["snmp_status"] = HOST_DOWN;

	/* if the first read did not work, loop until found */
	$snmp_sysObjectID = @snmpget($device["hostname"], $device["snmp_readstring"],
							".1.3.6.1.2.1.1.2.0", $device["snmp_timeout"]*1000,
							$device["snmp_retries"]);

	$snmp_sysObjectID = str_replace("enterprises", ".1.3.6.1.4.1", $snmp_sysObjectID);
	$snmp_sysObjectID = str_replace("OID: ", "", $snmp_sysObjectID);
	$snmp_sysObjectID = str_replace("iso", ".1", $snmp_sysObjectID);

	if (strlen($snmp_sysObjectID) > 0) {
		$snmp_sysObjectID = trim(str_replace("\"","", $snmp_sysObjectID));
		$host_up = TRUE;
		$device["snmp_status"] = HOST_UP;
	}else{
		/* loop through the default and then other common for the correct answer */
		$read_strings = explode(":",$device["snmp_readstrings"]);

		foreach($read_strings as $snmp_readstring) {
			$snmp_sysObjectID = @snmpget($device["hostname"], $snmp_readstring,
									".1.3.6.1.2.1.1.2.0", $device["snmp_timeout"]*1000,
									$device["snmp_retries"]);

			$snmp_sysObjectID = str_replace("enterprises", ".1.3.6.1.4.1", $snmp_sysObjectID);
			$snmp_sysObjectID = str_replace("OID: ", "", $snmp_sysObjectID);
			$snmp_sysObjectID = str_replace("iso", ".1", $snmp_sysObjectID);

			if (strlen($snmp_sysObjectID) > 0) {
				$snmp_sysObjectID = trim(str_replace("\"", "", $snmp_sysObjectID));
				$device["snmp_readstring"] = $snmp_readstring;
				$device["snmp_status"] = HOST_UP;
				$host_up = TRUE;
				break;
			}else{
				$device["snmp_status"] = HOST_DOWN;
				$host_up = FALSE;
			}
		}
	}

	if ($host_up) {
		$device["snmp_sysObjectID"] = $snmp_sysObjectID;

		/* get system description */
		$snmp_sysDescr = snmpget($device["hostname"], $device["snmp_readstring"],
							".1.3.6.1.2.1.1.1.0", $device["snmp_timeout"]*1000, $device["snmp_retries"]);
		if (strlen($snmp_sysDescr) > 0) {
			$snmp_sysDescr = trim(strtr($snmp_sysDescr,"\""," "));
			$device["snmp_sysDescr"] = $snmp_sysDescr;
		}

		/* get system uptime */
		$snmp_sysUptime = snmpget($device["hostname"], $device["snmp_readstring"],
							".1.3.6.1.2.1.1.3.0", $device["snmp_timeout"]*1000, $device["snmp_retries"]);
		if (strlen($snmp_sysUptime) > 0) {
			$snmp_sysUptime = trim(strtr($snmp_sysUptime,"\""," "));
			$device["snmp_sysUptime"] = $snmp_sysUptime;
		}
	}

	return $host_up;
}

/*	find_scanning_function - This function scans the mac_track_device_type database
  for a valid scanning function and then returns an array with the current device
  type and it's characteristics for the main mac_track_scanner function to call.
*/
function find_scanning_function(&$device, &$device_types) {
	$sysDescr_match = FALSE;
	$sysObjectID_match = FALSE;

	/* scan all device_types to determine the function to call */
	foreach($device_types as $device_type) {
		/* search for a matching snmp_sysDescr */
		if (substr_count($device_type["sysDescr_match"], "*") > 0) {
			/* need to assume mixed string */
			$parts = explode("*", $device_type["sysDescr_match"]);
			foreach($parts as $part) {
				if (substr_count($device["sysDescr_match"],$part) > 0) {
					$sysDescr_match = TRUE;
				}else{
					$sysDescr_match = FALSE;
				}
			}
		}else{
			if (substr_count($device["snmp_sysDescr"], $device_type["sysDescr_match"]) > 0) {
				$sysDescr_match = TRUE;
			}
		}

		/* search for a matching snmp_sysObjectID */
		if (substr_count($device_type["sysObjectID_match"], "*") > 0) {
			/* need to assume mixed string */
	  	$parts = explode("*", $device_type["sysObjectID_match"]);
			foreach($parts as $part) {
				if (substr_count($device["sysObjectID_match"],$part) > 0) {
					$sysObjectID_match = TRUE;
				}else{
					$sysObjectID_match = FALSE;
				}
			}
		}else{
			if (substr_count($device["snmp_sysObjectID"], $device_type["sysObjectID_match"]) > 0) {
				$sysObjectID_match = TRUE;
			}
		}

		if (($sysObjectID_match == TRUE) && ($sysDescr_match == TRUE)) {
			return $device_type;
		}
	}

	return array();
}

/*	get_arp_ip_ranges - This function reads a devices ARP table for a site and stores
  the IP address and MAC address combinations in the mac_track_ips table.
*/
function get_arp_ip_ranges($site, &$device) {
	global $debug, $scan_date;

	/* get the atifIndexes for the device */
	$atifIndexes = xform_indexed_data(".1.3.6.1.2.1.3.1.1.1", &$device, 6);
	debug("atifIndexes data collection complete");

	/* get the atPhysAddress for the device */
	$atPhysAddress = xform_indexed_data(".1.3.6.1.2.1.3.1.1.2", &$device, 6);

	/* convert the mac address if necessary */
	$keys = array_keys($atPhysAddress);
	$i = 0;
	foreach($atPhysAddress as $atAddress) {
		$atPhysAddress[$keys[$i]] = xform_mac_address($atAddress);
		$i++;
	}
	debug("atPhysAddress data collection complete");

	/* get the atPhysAddress for the device */
	$atNetAddress = xform_indexed_data(".1.3.6.1.2.1.3.1.1.3", &$device, 6);
	debug("atNetAddress data collection complete");

	/* get the ifNames for the device */
	$keys = array_keys($atifIndexes);
	$i = 0;
	foreach($atifIndexes as $atifIndex) {
		$atEntries[$i]["atifIndex"] = $atifIndex;
		$atEntries[$i]["atPhysAddress"] = $atPhysAddress[$keys[$i]];
		$atEntries[$i]["atNetAddress"] = $atNetAddress[$keys[$i]];
		$i++;
	}
	debug("atEntries assembly complete.");

	/* output details to database */
	foreach($atEntries as $atEntry) {
		$insert_string = "INSERT INTO mac_track_ips " .
			"(site_id,device_id,hostname,description,port_number," .
			"mac_address,ip_address,scan_date)" .
			" VALUES ('" .
			$device["site_id"] . "','" .
			$device["device_id"] . "','" .
			$device["hostname"] . "','" .
			$device["description"] . "','" .
			$atEntry["atifIndex"] . "','" .
			$atEntry["atPhysAddress"] . "','" .
			$atEntry["atNetAddress"] . "','" .
			$scan_date . "')";

		debug("SQL: " . $insert_string);

		db_execute($insert_string);
	}
    debug("HOST: " . $device["hostname"] . ", IP address information collection complete");

}

/*	get_generic_switch_ports - This is a basic function that will scan the dot1d
  OID tree for all switch port to MAC address association and stores in the
  mac_track_temp_ports table for future processing in the finalization steps of the
  scanning process.
*/
function get_generic_switch_ports($site, &$device, $lowPort = 0, $highPort = 0) {
	global $debug, $scan_date;

	/* initialize port counters */
	$device["ports_total"] = 0;
	$device["ports_active"] = 0;
	$device["ports_trunk"] = 0;

	/* get the ifIndexes for the device */
	$ifIndexes = xform_standard_indexed_data(".1.3.6.1.2.1.2.2.1.1", &$device);
	debug("ifIndexes data collection complete");

	/* get the ifTypes for the device */
	$ifTypes = xform_standard_indexed_data(".1.3.6.1.2.1.2.2.1.3", &$device);
	debug("ifTypes data collection complete.");

	/* get the ifNames for the device */
	$ifNames = xform_standard_indexed_data(".1.3.6.1.2.1.31.1.1.1.1", &$device);
	debug("ifNames data collection complete.");

	foreach($ifIndexes as $ifIndex) {
		$ifInterfaces[$ifIndex]["ifIndex"] = $ifIndex;
		$ifInterfaces[$ifIndex]["ifName"] = @$ifNames[$ifIndex];
		$ifInterfaces[$ifIndex]["ifType"] = $ifTypes[$ifIndex];
	}
	debug("ifInterfaces assembly complete.");

	get_base_dot1dTpFdbEntry_ports($site, &$device, &$ifInterfaces, "", TRUE, $lowPort, $highPort);
}

/*	get_generic_dot1dTpFdbEntry_ports - This function will grab information from the
	port bridge snmp table and return it to the calling progrem for further processing.
  This is a foundational function for all vendor data collection functions.
*/
function get_base_dot1dTpFdbEntry_ports($site, &$device, &$ifInterfaces, $snmp_readstring = "", $store_to_db = TRUE, $lowPort = 1, $highPort = 9999) {
	global $debug, $scan_date;

	/* initialize variables */
	$port_keys = array();
	$return_array = array();
	$new_port_key_array = array();
	$port_key_array = array();
	$port_number = 0;
	$ports_active = 0;
	$ports_total = 0;

	/* get additional information */
	snmp_set_quick_print(0);

	if (function_exists("snmp_set_valueretrieval")) {
		snmp_set_valueretrieval(SNMP_VALUE_PLAIN);
	}

	/* cisco uses a hybrid read string, if one is not defined, use the default */
	if ($snmp_readstring == "") {
	   $snmp_readstring = $device["snmp_readstring"];
	}

	/* get the operational status of the ports */
	$active_ports_array = xform_standard_indexed_data(".1.3.6.1.2.1.2.2.1.8", &$device);
	$indexes = array_keys($active_ports_array);

	$i = 0;
	foreach($active_ports_array as $port_info) {
		if ($ifInterfaces[$indexes[$i]]["ifType"] == 6) {
			if ($port_info == 1) {
				$ports_active++;
			}
			$ports_total++;
		}
		$i++;
	}

	if ($store_to_db) {
		print("INFO: HOST: " . $device["hostname"] . ", TYPE: " . substr($device["snmp_sysDescr"],0,40) . ", TOTAL PORTS: " . $ports_total . ", OPER PORTS: " . $ports_active);
		if ($debug) {
			print("\n");
		}

		$device["ports_active"] = $ports_active;
		$device["ports_total"] = $ports_total;
	}

	if ($ports_active > 0) {
		/* get bridge port to ifIndex mapping */
		$bridgePortIfIndexes = xform_standard_indexed_data(".1.3.6.1.2.1.17.1.4.1.2", $device, $snmp_readstring);

		/* map port status */
	    $port_status_array = array();
	    $port_status = mactrack_walk($device["hostname"], $snmp_readstring,
	                        ".1.3.6.1.2.1.17.4.3.1.3", $device["snmp_version"],
	                        $device["snmp_timeout"], $device["snmp_retries"]);

	    $i = 0;
	    foreach ($port_status as $port_state) {
	        $key = $port_state["oid"];
	        $key = str_replace("iso", ".1", $key);
	        $key = str_replace(".1.3.6.1.2.1.17.4.3.1.3.", "", $key);
	        $port_status_array[$i]["key"] = $key;
	        $port_status_array[$i]["status"] = $port_state["value"];

	        $i++;
	    }
	    $new_port_status = array_rekey($port_status_array, "key", "status");

	    /* map port numbers */
	    $port_numbers = mactrack_walk($device["hostname"], $snmp_readstring,
	                        ".1.3.6.1.2.1.17.4.3.1.2", $device["snmp_version"],
	                        $device["snmp_timeout"], $device["snmp_retries"]);

	    $i = 0;
	    foreach ($port_numbers as $port_number) {
	        if (($highPort == 0) ||
	            (($port_number["value"] >= $lowPort) &&
	            ($port_number["value"] <= $highPort))) {
	            $key = $port_number["oid"];
	            $key = str_replace("iso", ".1", $key);
	            $key = str_replace(".1.3.6.1.2.1.17.4.3.1.2.", "", $key);

	            if (@$new_port_status[$key] == "3") {
	                $port_key_array[$i]["key"] = $key;
	                $port_key_array[$i]["port_number"] = $port_number["value"];

	                $i++;
	            }
	        }
	    }

	    $i = 0;
	    foreach ($port_key_array as $port_key) {
	        /* map bridge port to interface port and check type */
	        if ($port_key["port_number"] > 0) {
	            if (sizeof($bridgePortIfIndexes) != 0) {
	                $brPortIfIndex = @$bridgePortIfIndexes[$port_key["port_number"]];
	                $brPortIfType = @$ifInterfaces[$brPortIfIndex]["ifType"];
	            }else{
	                $brPortIfIndex = $port_key["port_number"];
	                $brPortIfType = @$ifInterfaces[$port_key["port_number"]]["ifType"];
	            }

	            if (($brPortIfType == 6) &&
	                ($port_key["port_number"] <= $highPort) &&
	                ($port_key["port_number"] >= $lowPort)) { /* user ports - ethernet */
	                /* set some defaults  */
	                $new_port_key_array[$i]["vlan_id"] = "N/A";
	                $new_port_key_array[$i]["vlan_name"] = "N/A";
	                $new_port_key_array[$i]["mac_address"] = "NOT USER";
	                $new_port_key_array[$i]["port_number"] = "NOT USER";
	                $new_port_key_array[$i]["port_name"] = "N/A";

	                /* now set the real data */
	                $new_port_key_array[$i]["key"] = $port_key["key"];
	                $new_port_key_array[$i]["port_number"] = $port_key["port_number"];
	                $i++;
	            }
	        }
	    }
	    debug("Port number information collected.");

	    /* map mac address */
	    /* only continue if there were user ports defined */
	    if (sizeof($new_port_key_array) > 0) {
	        if ($device["snmp_version"] == 2) {
	            $port_macs = array();

	            $temp_array = mactrack_walk($device["hostname"], $snmp_readstring,
	                            ".1.3.6.1.2.1.17.4.3.1.1", $device["snmp_version"],
	                            $device["snmp_timeout"], $device["snmp_retries"]);

	            $i = 0;
	            foreach ($temp_array as $temp_value) {
	                $port_macs[$i]["oid"] = $temp_value["oid"];
	                $port_macs[$i]["oid"] = str_replace("iso", ".1", $port_macs[$i]["oid"]);
	                $port_macs[$i]["oid"] = str_replace(".1.3.6.1.2.1.17.4.3.1.1.", "", $port_macs[$i]["oid"]);
	                $port_macs[$i]["value"] = xform_mac_address($temp_value["value"]);
	                $i++;
	            }
	            $port_macs = array_rekey($port_macs, "oid", "value");

	            $i = 0;
	            foreach ($new_port_key_array as $port_key) {
	                $new_port_key_array[$i]["mac_address"] = @$port_macs[$port_key["key"]];

	                debug("INDEX: '". $i . "' MAC ADDRESS: " . $new_port_key_array[$i]["mac_address"]);

	                $i++;
	            }
	        }else{
	            $i = 0;
	            foreach ($new_port_key_array as $port_key) {
	                $OID = ".1.3.6.1.2.1.17.4.3.1.1." . $port_key["key"];
	                if ($device["snmp_version"] == 1) {
	                    $mac_address = @snmpget($device["hostname"], $snmp_readstring,
	                                        $OID, $device["snmp_timeout"]*1000, $device["snmp_retries"]);
	                }else{
	                    $mac_address = @snmp2_get($device["hostname"], $snmp_readstring,
	                                        $OID, $device["snmp_timeout"]*1000, $device["snmp_retries"]);
	                }

	                $new_port_key_array[$i]["mac_address"] = xform_mac_address($mac_address);

	                debug("INDEX: '". $i . "' MAC ADDRESS: " . $new_port_key_array[$i]["mac_address"]);

	                $i++;
	            }
	        }

	        debug("Port mac address information collected.");
	    }else{
	        debug("No user ports on this network.");
	    }
	}else{
        debug("No user ports on this network.");
    }


	if ($store_to_db) {
		if ($active_ports <= 0) {
			$device["last_runmessage"] = "Data collection completed ok";
		}elseif (sizeof($new_port_key_array) > 0) {
			$device["last_runmessage"] = "Data collection completed ok";
			db_store_device_port_results(&$device, $new_port_key_array, $scan_date);
		}else{
			$device["last_runmessage"] = "ERROR: Device encountered issues attempting to collect MAC data.";
			$device["snmp_status"] = HOST_ERROR;
		}

		if(!$debug) {
			print(" - Complete\n");
		}
	}else{
		return $new_port_key_array;
	}
}

/*	xform_mac_address - This function will take a variable that is either formated as
  hex or as a string representing hex and convert it to what the mactrack scanning
  function expects.
*/
function xform_mac_address($mac_address) {
    if (strlen($mac_address) == 0) {
        $mac_address = "NOT USER";
    }else{
        if (strlen($mac_address) > 10) { /* return is in ascii */
            $mac_address = trim(strtr($mac_address, "\"", ""));
            $mac_address = strtr($mac_address, " ", ":");
        }else{ /* return is hex */
            $mac = "";
            for ($j = 0; $j < strlen($mac_address); $j++) {
                $mac .= bin2hex($mac_address[$j]) . ":";
            }
            $mac_address = strtoupper($mac);
        }

        $mac_address = substr($mac_address,0,17);
    }

	return $mac_address;
}

/*	xform_standard_indexed_data - This function takes an OID, and a device, and
  optionally an alternate snmp_readstring as input parameters and then walks the
  OID and returns the data in array[index] = value format.
*/
function xform_standard_indexed_data($xformOID, &$device, $snmp_readstring = "") {
	/* get raw index data */
	if ($snmp_readstring == "") {
		$snmp_readstring = $device["snmp_readstring"];
	}

	$xformArray = mactrack_walk($device["hostname"], $snmp_readstring,
					$xformOID, $device["snmp_version"], $device["snmp_timeout"], $device["snmp_retries"]);

	$i = 0;
	foreach($xformArray as $xformItem) {
		$perPos = strrpos($xformItem["oid"], ".");
		$xformItemID = substr($xformItem["oid"], $perPos+1);
		$xformArray[$i]["oid"] = $xformItemID;
		$i++;
	}

	return array_rekey($xformArray, "oid", "value");
}

/*	xform_cisco_workgroup_port_data - This function is specific to Cisco devices that
  use the last two OID values from each complete OID string to represent the switch
  card and port.  The function returns data in the format array[card.port] = value.
*/
function xform_cisco_workgroup_port_data($xformOID, &$device) {
	/* get raw index data */
	$xformArray = mactrack_walk($device["hostname"], $device["snmp_readstring"],
							$xformOID, $device["snmp_version"], $device["snmp_timeout"], $device["snmp_retries"]);

	$i = 0;
	foreach($xformArray as $xformItem) {
		$perPos = strrpos($xformItem["oid"], ".");
		$xformItem_piece1 = substr($xformItem["oid"], $perPos+1);
		$xformItem_remainder = substr($xformItem["oid"], 0, $perPos);
		$perPos = strrpos($xformItem_remainder, ".");
		$xformItem_piece2 = substr($xformItem_remainder, $perPos+1);
		$xformArray[$i]["oid"] = $xformItem_piece2 . "/" . $xformItem_piece1;
		$i++;
	}

	return array_rekey($xformArray, "oid", "value");
}

/*	xform_indexed_data - This function is similar to other the other xform_* functions
  in that it takes the end of each OID and uses the last $xformLevel positions as the
  index.  Therefore, if $xformLevel = 3, the return value would be as follows:
  array[1.2.3] = value.
*/
function xform_indexed_data($xformOID, &$device, $xformLevel = 1) {
	/* get additional information */
	snmp_set_quick_print(0);

	if (function_exists("snmp_set_valueretrieval")) {
		snmp_set_valueretrieval(SNMP_VALUE_PLAIN);
	}

	/* get raw index data */
	$xformArray = mactrack_walk($device["hostname"], $device["snmp_readstring"],
						$xformOID, $device["snmp_version"], $device["snmp_timeout"], $device["snmp_retries"]);

	$i = 0;
	$output_array = array();
	foreach($xformArray as $xformItem) {
		/* break down key */
		$OID = $xformItem["oid"];
		for ($j = 0; $j < $xformLevel; $j++) {
			$perPos = strrpos($OID, ".");
			$xformItem_piece[$j] = substr($OID, $perPos+1);
			$OID = substr($OID, 0, $perPos);
		}

		/* reassemble key */
		$key = "";
		for ($j = $xformLevel-1; $j >= 0; $j--) {
			$key .= $xformItem_piece[$j];
			if ($j > 0) {
				$key .= ".";
			}
		}

		$output_array[$i]["key"] = $key;
		$output_array[$i]["value"] = $xformItem["value"];
		$i++;
	}

	return array_rekey($output_array, "key", "value");
}

/*	db_process_add - This function adds a process to the process table with the entry
  with the device_id as key.
*/
function db_process_add($device_id) {
	/* store pseudo process id in the database */
	db_execute("INSERT INTO mac_track_processes (process_id, status, start_date) VALUES ('" . $device_id . "', 'Running', NOW())");
}

/*	db_process_remove - This function removes a devices entry from the processes
  table indicating that the device is done processing and the next device may start.
*/
function db_process_remove($device_id) {
	db_execute("DELETE FROM mac_track_processes WHERE process_id='" . $device_id . "'");
}

/*	db_update_device_status - This function is used by the scanner to save the status
  of the current device including the number of ports, it's readstring, etc.
*/
function db_update_device_status(&$device, $host_up, $scan_date, $start_time) {
	global $debug;

	list($micro,$seconds) = split(" ", microtime());
	$end_time = $seconds + $micro;
	$runduration = $end_time - $start_time;

	if ($host_up == TRUE) {
		$update_string = "UPDATE mac_track_devices " .
			"SET ports_total='" . $device["ports_total"] . "'," .
			"vlans_total='" . $device["vlans_total"] . "'," .
			"ports_active='" . $device["ports_active"] . "'," .
			"ports_trunk='" . $device["ports_trunk"] . "'," .
			"snmp_readstring='" . $device["snmp_readstring"] . "'," .
			"snmp_sysObjectID='" . $device["snmp_sysObjectID"] . "'," .
			"snmp_sysDescr='" . $device["snmp_sysDescr"] . "'," .
			"snmp_sysUptime='" . $device["snmp_sysUptime"] . "'," .
			"snmp_status='" . $device["snmp_status"] . "'," .
			"last_runmessage='" . $device["last_runmessage"] . "'," .
			"last_rundate='" . $scan_date . "'," .
			"last_runduration='" . round($runduration,4) . "' " .
			"WHERE device_id ='" . $device["device_id"] . "'";
	}else{
		$update_string = "UPDATE mac_track_devices " .
			"SET snmp_status='" . $device["snmp_status"] . "'," .
			"last_runmessage='Device Unreachable', " .
			"last_rundate='" . $scan_date . "'," .
			"last_runduration='" . round($runduration,4) . "' " .
			"WHERE device_id ='" . $device["device_id"] . "'";
	}

	debug("SQL: " . $update_string);

	db_execute($update_string);
}

/*	db_store_device_results - This function stores each of the port results into
  the temporary port results table for future processes once all devices have been
  scanned.
*/
function db_store_device_port_results(&$device, $port_array, $scan_date) {
	global $debug;

	/* output details to database */
	foreach($port_array as $port_value) {
		if (($port_value["port_number"] <> "NOT USER") &&
			(($port_value["mac_address"] <> "NOT USER") && (strlen($port_value["mac_address"]) > 0))){
			$insert_string = "INSERT INTO mac_track_temp_ports " .
				"(site_id,device_id,hostname,description,vlan_id,vlan_name," .
				"mac_address,port_number,port_name,scan_date)" .
				" VALUES ('" .
				$device["site_id"] . "','" .
				$device["device_id"] . "','" .
				$device["hostname"] . "','" .
				$device["description"] . "','" .
				$port_value["vlan_id"] . "','" .
				$port_value["vlan_name"] . "','" .
				$port_value["mac_address"] . "','" .
				$port_value["port_number"] . "','" .
				$port_value["port_name"] . "','" .
				$scan_date . "')";

			debug("SQL: " . $insert_string);

			db_execute($insert_string);
		}
	}
}

/*	mactrack_walk - This utility function performs either a standard walk or a bulk
  walk depending on the snmp_version of the host.  It is more advantageous to utilize
  bulk methods as they are much more efficient.
*/
function mactrack_walk($hostname, $snmp_readstring, $OID, $snmp_version, $snmp_timeout, $snmp_retries, $repair_hex = FALSE) {
	if ($snmp_version == 1) {
		$return_array = cacti_snmp_walk($hostname, $snmp_readstring,
					$OID, $snmp_version,
					"", "", 161, $snmp_timeout);
	}else{
		$path_snmpbulkwalk = read_config_option("path_snmpbulkwalk");
		$snmp_version = "2c";
		$return_array = array();
   		$exec_string = trim($path_snmpbulkwalk .
   		" -c " . $snmp_readstring .
			" -OnUQ -v " . $snmp_version .
			" -r " . $snmp_retries .
			" -t " . ceil($snmp_timeout/1000) . " -Ccr50 " .
			$hostname . " " . $OID);

		exec($exec_string, &$temp_array, $return_code);

		$i = 0;
		foreach ($temp_array as $return_value) {
			/* fix nasty snmpv2 bugs in Cisco 4006 switches */
			if (!(substr_count($return_value, "=") == 0)) {
				list($oid, $value) = explode("=", $return_value);
				$oid = str_replace("iso", ".1", $oid);
				$return_array[$i]["oid"] = trim($oid);
				$return_array[$i]["value"] = trim(str_replace("\"", "", $value));
				$i++;
			}
		}
	}

	return $return_array;
}

?>