#!/usr/bin/php -q
<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2005 Larry Adams                                          |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
*/

/* do NOT run this script through a web browser */
if (!isset($_SERVER["argv"][0])) {
	die("<br><strong>This script is only meant to run at the command line.</strong>");
}

/* We are not talking to the browser */
$no_http_headers = true;

/* Start Initialization Section */
include(dirname(__FILE__) . "/include/config.php");
include_once($config["base_path"] . "/lib/poller.php");

/* get the mactrack polling cycle */
$max_run_duration = read_config_option("mt_collection_timing");

if (is_numeric($max_run_duration)) {
	/* let PHP a 5 minutes less than the rerun frequency */
	$max_run_duration = ($max_run_duration * 60) - 300;
	ini_set("max_execution_time", $max_run_duration);
}

/* get the max script runtime and kill old scripts */
$max_script_runtime = read_config_option("mt_script_runtime");
$delete_time = date("Y-m-d H:i:s", strtotime("-" . $max_script_runtime . " Minutes"));
db_execute("delete from mac_track_processes where start_date < '" . $delete_time . "'");

/* Disable Mib File Loading */
putenv("MIBS=RFC-1215");

if (read_config_option("mt_collection_timing") != "disabled") {
	/* initialize variables */
	$site_id = "";

	/* process calling arguments */
	$parms = $_SERVER["argv"];
	array_shift($parms);

	$debug = FALSE;

	foreach($parms as $parameter) {
		@list($arg, $value) = @explode("=", $parameter);

		switch ($arg) {
		case "-sid":
			$site_id = $value;
			break;
		case "-d":
			$debug = TRUE;
			break;
		case "-h":
			display_help();
			exit;
		case "-v":
			display_help();
			exit;
		case "--version":
			display_help();
			exit;
		case "--help":
			display_help();
			exit;
		default:
			print "ERROR: Invalid Parameter " . $parameter . "\n\n";
			display_help();
			exit;
		}
	}

	$seconds_offset = read_config_option("mt_collection_timing");
	if ($seconds_offset <> "disabled") {
		$seconds_offset = $seconds_offset * 60;
		/* find out if it's time to collect device information */
		$base_start_time = read_config_option("mt_base_time");
		$database_maint_time = read_config_option("mt_maint_time");
		$last_run_time = read_config_option("mt_last_run_time");
		$last_db_maint_time = read_config_option("mt_last_db_maint_time");
		$previous_base_start_time = read_config_option("mt_prev_base_time");
		$previous_db_maint_time = read_config_option("mt_prev_db_maint_time");

		if (!empty($previous_base_start_time)) {
			if ($base_start_time <> $previous_base_start_time) {
				debug("Detected that user changed the start time\n");
				unset($last_run_time);
				db_execute("DELETE FROM settings WHERE name='mt_last_run_time'");
			}
		}

		if (!empty($previous_db_maint_time)) {
			if ($prev_db_maint_time <> $database_maint_time) {
				debug("Detected that user changed the db maintenance time\n");
				unset($last_db_maint_time);
				db_execute("DELETE FROM settings WHERE name='mt_last_db_maint_time'");
			}
		}

		/* set to detect if the user cleared the time between polling cycles */
		db_execute("REPLACE INTO settings (name, value) VALUES ('mt_prev_base_time', '$base_start_time')");

		$current_time = strtotime("now");

		if (empty($last_run_time)) {
			$collection_never_completed = TRUE;
			if ($current_time > strtotime($base_start_time)) {
				/* if timer expired within a polling interval, then poll */
				if (($current_time - 300) < strtotime($base_start_time)) {
					$next_run_time = strtotime(date("Y-m-d") . " " . $base_start_time);
    			}else{
					$next_run_time = strtotime(date("Y-m-d") . " " . $base_start_time) + 3600*24;
				}
			}else{
				$next_run_time = strtotime(date("Y-m-d") . " " . $base_start_time);
			}
		}else{
			$collection_never_completed = FALSE;
			$next_run_time = $last_run_time + $seconds_offset;
		}

		if (empty($last_db_maint_time)) {
			if (strtotime($base_start_time) < $current_time) {
				$next_db_maint_time = strtotime(date("Y-m-d") . " " . $database_maint_time) + 3600*24;
			}else{
				$next_db_maint_time = strtotime(date("Y-m-d") . " " . $database_maint_time);
			}
		}else{
			$next_db_maint_time = $last_db_maint_time + 24*3600;
		}

		$time_till_next_run = $next_run_time - $current_time;
		$time_till_next_db_maint = $next_db_maint_time - $current_time;

		if ($time_till_next_run < 0) {
			/* take time and log performance data */
			list($micro,$seconds) = split(" ", microtime());
			$start = $seconds + $micro;

			db_execute("REPLACE INTO settings (name, value) VALUES ('mt_last_run_time', '$current_time')");
			$running_processes = db_fetch_cell("SELECT count(*) FROM mac_track_processes");

			if ($running_processes) {
				cacti_log("ERROR: Can not start MAC Trackinging process.  There is already one in progress\n");
			}else{
				collect_mactrack_data($start, $site_id);
				log_mactrack_statistics("collect");
			}
		}

		if ($time_till_next_db_maint < 0) {
			/* take time and log performance data */
			list($micro,$seconds) = split(" ", microtime());
			$start = $seconds + $micro;

			db_execute("REPLACE INTO settings (name, value) VALUES ('mt_last_db_maint_time', '$current_time')");
			perform_mactrack_db_maint();
			log_mactrack_statistics("maint");
		}
	}
}

/*	display_help - displays the usage of the function */
function display_help () {
	print "Network Mac Tracker Master Process Control Version 1.0, Copyright 2005 - Larry Adams\n\n";
	print "usage: poller_mactrack.php [-sid=site_id] [-d] [-h] [--help] [-v] [--version]\n\n";
	print "-sid=site_id  - the mac_track_sites site_id to scan\n";
	print "-d            - Display verbose output during execution\n";
	print "-v --version  - Display this help message\n";
	print "-h --help     - display this help message\n";
}

function debug($message) {
	global $debug;

	if ($debug) {
		print("DEBUG: " . $message . "\n");
	}
}

function collect_mactrack_data($start, $site_id = 0) {
	global $max_run_duration, $config, $debug;

	/* reset the processes table */
	db_execute("TRUNCATE TABLE mac_track_processes");

	/* retreive the number of concurrent mac_track processes to run */
	/* default to 10 for now */
	$concurrent_processes = read_config_option("mt_processes");

	/* get php binary path */
	$command_string = read_config_option("path_php_binary");

	/* save the scan date information */
	$scan_date = date("Y-m-d H:i:s");
	db_execute("REPLACE INTO settings (name, value) VALUES ('mactrack_scan_date', '$scan_date')");

	/* just in case we've run too long */
	$exit_mactrack = TRUE;

	/* start mainline processing, order by site_id to keep routers grouped with switches */
	if ($site_id > 0) {
		$device_ids = db_fetch_assoc("SELECT device_id FROM mac_track_devices WHERE site_id='" . $site_id . "'");
	}else{
		$device_ids = db_fetch_assoc("SELECT device_id FROM mac_track_devices ORDER BY site_id");
	}

	if (sizeof($device_ids) > 0) {
		/* scan through all devices */
		$j = 0;
		$i = 0;
		$processes_available = $concurrent_processes;
		while (($i+$j) < sizeof($device_ids)) {
			for ($i = 0; $i < $processes_available; $i++) {
				if (($j+$i) >= sizeof($device_ids)) break;

				$extra_args = " -q " . $config["base_path"] . "/mactrack_scanner.php -id=" . $device_ids[$i+$j]["device_id"];
				debug("CMD: " . $command_string . $extra_args);
				exec_background($command_string, $extra_args);
			}
			$j = $j + $i;

			if ($i > 0) {
				debug("A process launch just occurred.");
			}

			/* wait 2 seconds for proccesses to exit */
			sleep(2);

			$processes_running = db_fetch_cell("SELECT count(*) FROM mac_track_processes");
			$processes_available = $concurrent_processes - $processes_running;

			/* take time to check for an exit condition */
			list($micro,$seconds) = split(" ", microtime());
			$current = $seconds + $micro;

			/* exit if we've run too long */
			if (($current - $start) > $max_run_duration) {
				$exit_mactrack = TRUE;
				cacti_log("ERROR: MacTracking timed out during main script processing.\n");
				break;
			}
		}

		/* wait for last process to exit */
		$processes_running = db_fetch_cell("SELECT count(*) FROM mac_track_processes");
		while (($processes_running > 0) && (!$exit_mactrack)) {
			$processes_running = db_fetch_cell("SELECT count(*) FROM mac_track_processes");
			sleep(2);

			/* take time to check for an exit condition */
			list($micro,$seconds) = split(" ", microtime());
			$current = $seconds + $micro;

			/* exit if we've run too long */
			if (($current - $start) > $max_run_duration) {
				$exit_mactrack = TRUE;
				cacti_log("ERROR: MacTracking timed out during main script processing.\n");
				break;
			}

			debug("Waiting on " . $processes_running . " to complete prior to exiting.");
		}

		/* update IP information into the mac_track_temp_ports table */
		db_execute("UPDATE mac_track_temp_ports
					INNER JOIN mac_track_ips
					ON (mac_track_temp_ports.mac_address = mac_track_ips.mac_address
					AND mac_track_temp_ports.scan_date = mac_track_ips.scan_date
					AND mac_track_temp_ports.site_id = mac_track_ips.site_id)
					SET mac_track_temp_ports.ip_address = mac_track_ips.ip_address") . "- IP addresses transferred";
		debug("Finished associating IP addresses with MAC addresses.");

		/* transfer temp port results into permanent table */
		db_execute("INSERT INTO mac_track_ports
					(site_id, device_id, hostname, description,
					vlan_id, vlan_name, mac_address, ip_address,
					port_number, port_name, scan_date)
					SELECT site_id, device_id, hostname, description,
					vlan_id, vlan_name, mac_address, ip_address,
					port_number, port_name, scan_date
					FROM mac_track_temp_ports");
		debug("Finished transferring scan results to main table.");

		/* collect statistics */
		if ($site_id == 0) {
			$stats = db_fetch_assoc("SELECT site_id,
							count(device_id) as total_devices,
							sum(ports_active) as total_macs,
							sum(ports_total) as total_user_ports
							FROM mac_track_devices
							GROUP BY site_id");
		}else{
			$stats = db_fetch_assoc("SELECT site_id,
							count(device_id) as total_devices,
							sum(ports_active) as total_macs,
							sum(ports_total) as total_user_ports
							FROM mac_track_devices
							WHERE site_id='$site_id'
							GROUP BY site_id");
		}

		/* collect total device errors */
		$errors = db_fetch_assoc("SELECT site_id, snmp_status, count(device_id) as total_device_errors
						FROM mac_track_devices
						GROUP BY site_id, snmp_status");

		$ips = array_rekey(db_fetch_assoc("SELECT site_id, count(ip_address) as total_ips
						FROM mac_track_ips
						GROUP BY site_id"), "site_id", "total_ips");

		foreach($errors as $error) {
			if (!isset($error_count[$error["site_id"]])) {
				$error_count[$error["site_id"]] = 0;
			}
			if ($error["snmp_status"] <> 3) {
				$error_count[$error["site_id"]] += $error["total_device_errors"];
			}
		}

		foreach($stats as $stat) {
			$num_ips = @$ips[$stat["site_id"]];
			if (empty($num_ips)) $num_ips = 0;

		    $update_string = "UPDATE mac_track_sites SET " .
				"total_devices='" . $stat["total_devices"] . "', " .
				"total_ips='" . $num_ips . "', " .
				"total_macs='" . $stat["total_macs"] . "', " .
				"total_user_ports='" . $stat["total_user_ports"] . "', " .
				"total_device_errors='" . $error_count[$stat["site_id"]] . "' " .
				"WHERE site_id='" . $stat["site_id"] . "'";

			db_execute($update_string);
		}
		debug("Finished updating site table with collection statistics.");

		/* purge the ip address and temp port table */
		db_execute("TRUNCATE TABLE mac_track_temp_ports");
		db_execute("TRUNCATE TABLE mac_track_ips");
	}else{
		cacti_log("NOTE: MACTRACK has no devices to process at this time\n");
	}
}

function perform_mactrack_db_maint() {
	/* remove stale records from the poller database */
	$retention = read_config_option("mt_data_retention");
	switch ($retention) {
	case "2days":
		$retention_date = date("Y-m-d H:i:s", strtotime("-2 Days"));
		break;
	case "5days":
		$retention_date = date("Y-m-d H:i:s", strtotime("-5 Days"));
		break;
	case "1week":
		$retention_date = date("Y-m-d H:i:s", strtotime("-1 Week"));
		break;
	case "3weeks":
		$retention_date = date("Y-m-d H:i:s", strtotime("-3 Weeks"));
		break;
	case "1month":
		$retention_date = date("Y-m-d H:i:s", strtotime("-1 Month"));
		break;
	case "2months":
		$retention_date = date("Y-m-d H:i:s", strtotime("-2 Months"));
		break;
	case "2weeks":
		$retention_date = date("Y-m-d H:i:s", strtotime("-2 Weeks"));
		break;
	default:
		$retention_date = date("Y-m-d H:i:s", strtotime("-2 Days"));
	}

	debug("Started deleting old records from the main database.");
	db_execute("DELETE QUICK FROM mac_track_ports WHERE scan_date < '$retention_date'");
	db_execute("OPTIMIZE TABLE mac_track_ports");
	debug("Finished deleting old records from the main database.");
}

function log_mactrack_statistics($type = "collect") {
	global $start, $site_id;

	/* let's get the number of devices */
	if (is_numeric($site_id)) {
		$devices = db_fetch_cell("SELECT Count(*) FROM mac_track_devices WHERE site_id='$site_id'");
	}else{
		$devices = db_fetch_cell("SELECT Count(*) FROM mac_track_devices");
	}

	/* take time and log performance data */
	list($micro,$seconds) = split(" ", microtime());
	$end = $seconds + $micro;

	if ($type == "collect") {
		$cacti_stats = sprintf(
			"Time:%01.4f " .
			"ConcurrentProcesses:%s " .
			"Devices:%s ",
			round($end-$start,4),
			$concurrent_processes,
			$devices);

		cacti_log("MACTRACK COLLECTION STATS: " . $cacti_stats ,true,"SYSTEM");
	}else{
		$cacti_stats = sprintf("Time:%01.4f", round($end-$start,4));

		cacti_log("MACTRACK DB MAINT STATS: " . $cacti_stats ,true,"SYSTEM");
	}
}

?>